﻿using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.JavaScript;
using Syncfusion.JavaScript.DataSources;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WebApplication11.Data;
using WebApplication11.Models;

namespace WebApplication11.Controllers
{
	public class BaseController : Controller
	{
		public DataContext DB { get; private set; }

		public BaseController(DataContext context, UserManager<ApplicationUser> usermanager, RoleManager<IdentityRole> rolemanager)
		{
			DB = context;
			this.UserManager = usermanager;
			this.RoleManager = rolemanager;
		}

		public UserManager<ApplicationUser> UserManager { get; private set; }
		public RoleManager<IdentityRole> RoleManager { get; private set; }

		public async Task<ApplicationUser> CurrentUser()
		{
			if (User.Identity.Name == null)
				return null;
			return await UserManager.FindByNameAsync(User.Identity.Name);
		}
	}

	public class BaseController<M, K> : BaseController
		where M : class, IModel<M, K>, new()
	{
		public BaseController(DataContext context, UserManager<ApplicationUser> usermanager, RoleManager<IdentityRole> rolemanager) : base(context, usermanager, rolemanager)
		{
			SetDataSourceHandler();
		}

		public virtual void SetDataSourceHandler()
		{
			DataSourceHandler = new DataSourceHandler<M, K>() { Controller = this };
		}

		public ActionResult DataSource(DataManager dm)
		{
			return DataSourceHandler.DataSource(dm);
		}

		public virtual async Task<IActionResult> Update(M value)
		{
			return await DataSourceHandler.UpdateAsync(value);
		}

		public virtual async Task<IActionResult> Insert(M value)
		{
			return await DataSourceHandler.InsertAsync(value);
		}

		public virtual async Task<IActionResult> Remove(K key)
		{
			return await DataSourceHandler.RemoveAsync(key);
		}

		public DataSourceHandler<M, K> DataSourceHandler { get; set; }
	}

}