﻿using Microsoft.AspNetCore.Mvc;
using Syncfusion.JavaScript;
using Syncfusion.JavaScript.DataSources;
using Syncfusion.Linq;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using WebApplication11.Controllers;
using WebApplication11.Models;

namespace WebApplication11.Data
{
	public class ResultParams<T, K>
	{
		public K key { get; set; }
		public string action { get; set; }
		public List<T> added { get; set; }

		public List<T> changed { get; set; }
		public List<T> deleted { get; set; }
		public T value { get; set; }
	}

	public class DataResult
	{
		public IEnumerable result { get; set; }
		public IEnumerable aggregate { get; set; }
		public int count { get; set; }
	}


	public class DataSourceHandler<M, K>
		where M : class, IModel<M, K>, new()
	{
		public BaseController Controller { get; set; }

		public virtual IEnumerable GetAllRecords()
		{
			return DB.Set<M>();
		}

		public DataContext DB { get { return Controller.DB; } }

		public ActionResult DataSource(DataManager dm)
		{
			DataResult result = new DataResult();
			DataOperations operation = new DataOperations();

			IEnumerable set = GetAllRecords();
			if (dm.Where != null && dm.Where.Count > 0)
				set = operation.PerformWhereFilter(set, dm.Where, dm.Where[0].Operator);
            result.count = 300;
			if (dm.Aggregates != null)
			{
				List<string> str = new List<string>();
				for (var i = 0; i < dm.Aggregates.Count; i++)
					str.Add(dm.Aggregates[i].Field);
				result.aggregate = operation.PerformSelect(set, str);
			}
			if (dm.Sorted != null)
				set = operation.PerformSorting(set, dm.Sorted);
			if (dm.Skip > 0)
				set = operation.PerformSkip(set, dm.Skip);
			if (dm.Take > 0)
				set = operation.PerformTake(set, dm.Take);
			List<M> list = new List<M>();
			foreach (M r in set)
				list.Add(r);
			result.result = list;
			foreach (var r in result.result)
				Debug.WriteLine("{0}", r);
			return Controller.Json(result);
		}

		public virtual async Task<IActionResult> UpdateAsync(M value)
		{
			DB.Update(value);
			await DB.SaveChangesAsync();
			return Controller.Json(value);
		}

		public virtual async Task<IActionResult> InsertAsync(M value)
		{
			DB.Add(value);
			await DB.SaveChangesAsync();
			return Controller.Json(value);
		}

		public virtual async Task<IActionResult> RemoveAsync(K key)
		{
			var result = await DB.Set<M>().FindAsync(key);
			DB.Remove(result);
			await DB.SaveChangesAsync();
			return Controller.Json(result);
		}
	}
}
